@@ -30,7 +30,7 @@ from drawBot.context.baseContext import BezierPath
 from pagebot.constants import (DEFAULT_FILETYPE, DEFAULT_FONT, LEFT, RIGHT,
         CENTER, FILETYPE_PDF, FILETYPE_SVG, FILETYPE_PNG, FILETYPE_JPG,
         FILETYPE_GIF, FILETYPE_MOV, SCALE_TYPE_FITWH, SCALE_TYPE_FITW,
-        SCALE_TYPE_FITH, DEFAULT_FALLBACK_FONT_PATH, ORIGIN)
+        SCALE_TYPE_FITH, DEFAULT_FALLBACK_FONT_PATH, ORIGIN, EXPORT)
 from pagebot.contexts.basecontext.babelstring import BabelString
 from pagebot.contexts.basecontext.babelrun import BabelLineInfo, BabelRunInfo
 from pagebot.contexts.basecontext.basecontext import BaseContext
@@ -52,7 +52,7 @@ class DrawBotContext(BaseContext):
             FILETYPE_GIF, FILETYPE_MOV)
 
     '''
-    /_scaled will be ignored with default .gitignore settings.  If the
+    /_scaled will be ignored with default .gitignore settings. If the
     docs/images/_scaled folder need to be committed to a Git repository, remove
     _scaled from .gitignore.
     '''
@@ -81,7 +81,7 @@ class DrawBotContext(BaseContext):
     # Drawing.
 
     def newDrawing(self, w=None, h=None, doc=None):
-        """Clear output canvas, start new export file. DrawBot function.
+        """Clears the canvas, starts a file.
 
         >>> context = DrawBotContext()
         >>> context.newDrawing()
@@ -89,17 +89,26 @@ class DrawBotContext(BaseContext):
         self.b.newDrawing()
 
     def endDrawing(self, doc=None):
+        """
+        >>> context = DrawBotContext()
+        >>> context.endDrawing()
+        """
         self.b.endDrawing()
 
+
+    # Saving / export.
+
     def saveDrawing(self, path, multiPage=None):
-        """Select non-standard DrawBot export builders here. Save the current
-        image as path, rendering depending on the extension of the path file.
-        In case the path starts with "_export", create its directories.
+        """Save the current image as path, rendering depending on the
+        extension. In case the path starts with "_export", optionally creates
+        directories if they don't exist yet.
 
         >>> context = DrawBotContext()
         >>> context.saveImage('_export/PageBotContext-saveDrawing.pdf')
-
         """
+        #if not multiPage:
+        #    multiPage = True
+
         self.checkExportPath(path)
 
         if path.lower().endswith('.mov'):
@@ -109,13 +118,63 @@ class DrawBotContext(BaseContext):
 
     saveImage = saveDrawing
 
+    def export(self, fileName, folderName=None, extension=None):
+        """Saves file to filename with default folder name and extension."""
+        if not folderName:
+            folderName = EXPORT
+        if not extension:
+            extension = 'pdf'
+
+        path = '%s/%s.%s' % (folderName, fileName, extension)
+        self.saveImage(path)
+
     def getDrawing(self):
-        """Returns a PDF document of the current state."""
+        """Returns a PDF document of the current state.
+
+
+        FIXME: should return a drawing object, need separate function for PDF.
+        """
         return self.b.pdfImage()
 
     def setStyles(self, styles):
         pass
 
+    # Magic variables.
+
+    def _get_width(self):
+        return self.b.width()
+
+    def _get_height(self):
+        return self.b.height()
+
+    def sizes(self, paperSize=None):
+        return self.b.sizes(paperSize=paperSize)
+
+    def pageCount(self):
+        return self.b.pageCount()
+
+    # Public callbacks.
+
+    def setSize(self, width, height=None):
+        # Same as setSize?
+        return self.b.size(width, height=height)
+
+    def newPage(self, w=None, h=None, doc=None, **kwargs):
+        """Creates a new page. takes document size, w and h, or 1000 x 1000px
+        if no arguments are provided.
+
+        >>> from pagebot.toolbox.units import px
+        >>> from pagebot import getContext
+        >>> context = getContext()
+        >>> context.newPage(pt(100), pt(100))
+        >>> context.newPage(100, 100)
+        """
+        if doc is not None:
+            w = w or doc.w
+            h = h or doc.h
+        wpt, hpt = upt(w, h)
+        self.b.newPage(wpt, hpt)
+
     # Graphic state.
 
     def save(self):
@@ -311,28 +370,59 @@ class DrawBotContext(BaseContext):
         return fs
 
     def text(self, s, p, align=None):
+        """
+        >>> context = DrawBotContext()
+        >>> s = 'test'
+        >>> p = (0, 0)
+        >>> context.text(s, p)
+        >>> bs = context.newString(s)
+        >>> context.text(bs, p)
+        """
+
         if isinstance(s, str):
             s = self.newString(s)
         assert isinstance(s, BabelString)
+
         if isinstance(p, BezierPath):
             self.textPath(s, p, align=align)
         else:
             self.b.text(s.cs, p, align=align)
 
     def textBox(self, s, box):
+        """
+        >>> context = DrawBotContext()
+        >>> s = 'test'
+        >>> box = (0, 0, 100, 100)
+        >>> context.textBox(s, box)
+        >>> bs = context.newString(s)
+        >>> context.textBox(bs, box)
+        """
+
         if isinstance(s, str):
             s = self.newString(s)
         assert isinstance(s, BabelString)
         self.b.textBox(s.cs, box, align=None)
 
     def textPath(self, s, p, align=None):
+        """
+        >>> from pagebot.fonttoolbox.objects.font import findFont
+        >>> context = DrawBotContext()
+        >>> f = findFont('Roboto-Regular')
+        >>> g = f['H']
+        >>> s = 'test'
+        >>> path = context.getGlyphPath(g)
+        >>> context.textPath(s, path)
+        """
+        if isinstance(s, str):
+            s = self.newString(s)
+        assert isinstance(s, BabelString)
         p.text(s.cs) # font='', fontSize='')
         self.b.fill(0)
         self.b.textBox(s.cs, p)
 
     def textSize(self, bs, w=None, h=None, align=None, ascDesc=False):
-        """Answers the width and height of the native @fs formatted string
-        with an optional given w or h.
+        """Answers the width and height of the native @fs formatted string with
+        an optional given w or h.
 
         >>> from pagebot.document import Document
         >>> from pagebot.elements import *
@@ -408,6 +498,10 @@ class DrawBotContext(BaseContext):
     #
 
     def newPath(self):
+        """
+        >>> context = DrawBotContext()
+        >>> path = context.newPath()
+        """
         # TODO: use our own Bézier path.
         self._bezierpath = self.b.BezierPath()
         #self._bezierpath = BaseBezierPath()
@@ -422,8 +516,6 @@ class DrawBotContext(BaseContext):
         >>> from pagebot.fonttoolbox.objects.font import findFont
         >>> context = DrawBotContext()
         >>> f = findFont('Roboto-Regular')
-        >>> print(f)
-        <Font Roboto-Regular>
         >>> g = f['H']
         >>> path = context.getGlyphPath(g)
         """
@@ -469,12 +561,26 @@ class DrawBotContext(BaseContext):
         return path._path.bezierPathByFlatteningPath()
 
     def getFlattenedPath(self, path=None):
+        """
+        >>> from pagebot.fonttoolbox.objects.font import findFont
+        >>> context = DrawBotContext()
+        >>> f = findFont('Roboto-Regular')
+        >>> g = f['H']
+        >>> path = context.getGlyphPath(g)
+        >>> path = context.getFlattenedPath(path)
+        """
         return self.bezierPathByFlatteningPath(path=path)
 
     def getFlattenedContours(self, path=None):
         """Answers the flattened Bézier path as  a contour list [contour,
         contour, ...] where contours are lists of point2D() points.
 
+        >>> from pagebot.fonttoolbox.objects.font import findFont
+        >>> context = DrawBotContext()
+        >>> f = findFont('Roboto-Regular')
+        >>> g = f['H']
+        >>> path = context.getGlyphPath(g)
+        >>> path = context.getFlattenedContours(path)
         """
         contour = []
         flattenedContours = [contour]
@@ -498,11 +604,23 @@ class DrawBotContext(BaseContext):
         return flattenedContours
 
     def onBlack(self, p, path=None):
-        """Answers if the single point (x, y) is on black."""
+        """Answers if the single point (x, y) is on black.
+
+        >>> from pagebot.fonttoolbox.objects.font import findFont
+        >>> context = DrawBotContext()
+        >>> f = findFont('Roboto-Regular')
+        >>> g = f['H']
+        >>> path = context.getGlyphPath(g)
+        >>> p = (0, 0)
+        >>> context.onBlack(p, path=path)
+        False
+        >>> # TODO: find out coordinate that is inside 'H'.
+        >>> #context.onBlack((120, 10), path=path)
+        """
         if path is None:
             path = self.bezierpath
         p = point2D(p)
-        return path._bezierpath.containsPoint_(p)
+        return path._path.containsPoint_(p)
 
 
     # Path drawing behavior.
@@ -512,7 +630,7 @@ class DrawBotContext(BaseContext):
 
         >>> from pagebot.toolbox.units import pt, mm
         >>> context = DrawBotContext()
-        >>> context.newDrawing()
+        >>> #context.newDrawing()
         >>> context.newPage(420, 420)
         >>> context.setStrokeWidth(pt(0.5))
         >>> context.setStrokeWidth(mm(0.5))
@@ -841,7 +959,7 @@ class DrawBotContext(BaseContext):
         ``def openTypeFeatures(self, *args, **features):``
 
         >>> context = DrawBotContext()
-        >>> context.newDrawing()
+        >>> #context.newDrawing()
         >>> context.newPage(420, 420)
         >>> context.openTypeFeatures(dict(smcp=True, zero=True))
         """
